"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = cssToJss;
exports.collectAnimationNames = collectAnimationNames;
exports.splitGlobals = splitGlobals;
exports.convertSelectors = convertSelectors;
exports.convertAnimationNames = convertAnimationNames;
exports.convertNodes = convertNodes;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _jscodeshift = _interopRequireDefault(require("jscodeshift"));

var recast = _interopRequireWildcard(require("recast"));

var postcss = _interopRequireWildcard(require("postcss"));

var _camelCase = _interopRequireDefault(require("lodash/camelCase"));

var _upperFirst = _interopRequireDefault(require("lodash/upperFirst"));

var _defaultUnits = _interopRequireDefault(require("./defaultUnits"));

var _postcssSelectorParser = _interopRequireDefault(require("postcss-selector-parser"));

function cssToJss(fileInfo, api, options) {
  var selectionStart = options.selectionStart ? parseInt(options.selectionStart) : 0;
  var selectionEnd = options.selectionEnd ? parseInt(options.selectionEnd) : fileInfo.source.length;
  var before = fileInfo.source.substring(0, selectionStart);
  var rawCss = fileInfo.source.substring(selectionStart, selectionEnd);
  var after = fileInfo.source.substring(selectionEnd);
  var root = postcss.parse(rawCss);
  var converted = convertNodes(root.nodes || []);
  convertSelectors(converted);
  var animationNames = collectAnimationNames(root.nodes || []);
  convertAnimationNames(converted, animationNames);
  convertFallbacks(converted);
  var rawConverted = converted.properties.map(function (p) {
    return recast.prettyPrint(p).code;
  }).join(',\n');
  return "".concat(before).concat(rawConverted).concat(after);
}

function collectAnimationNames(nodes) {
  var names = new Set();
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = nodes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _node = _step.value;
      if (_node.type !== 'atrule') continue;
      var name = _node.name,
          params = _node.params;
      if (name !== 'keyframes') continue;
      var match = /^\S+/.exec(params);
      if (match) names.add(match[0]);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return names;
}

var getRawKey = function getRawKey(key) {
  return key.type === 'Identifier' ? key.name : key.type === 'StringLiteral' ? key.value : null;
};

var processDeep = function processDeep(iteratee) {
  return function (node) {
    var handle = function handle(node) {
      iteratee(node);
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = node.properties[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var prop = _step2.value;
          if (prop.type !== 'ObjectProperty') continue;
          if (prop.value.type === 'ObjectExpression') handle(prop.value);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
            _iterator2["return"]();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    };

    handle(node);
  };
};

var baseSelectorParser = (0, _postcssSelectorParser["default"])();

var parseSelector = function parseSelector(selector) {
  return baseSelectorParser.astSync(selector);
};

function splitGlobals(selector) {
  var ast = parseSelector(selector);
  var locals = [];
  var globals = [];
  ast.each(function (node) {
    if (node.type === 'selector' && node.first.type !== 'class' && node.first.type !== 'nesting' && (node.first.type !== 'tag' || !node.first.value.startsWith('@'))) globals.push(node);else locals.push(node);
  });
  return [locals.length ? _postcssSelectorParser["default"].root({
    nodes: locals
  }).toString() : null, globals.length ? _postcssSelectorParser["default"].root({
    nodes: globals
  }).toString() : null];
}

function convertSelectors(root) {
  var replacements = new Map();
  var processRootSelector = (0, _postcssSelectorParser["default"])(function (root) {
    var _first$first;

    if (root.length !== 1) return;
    var first = root.first;

    if ((first === null || first === void 0 ? void 0 : first.type) !== 'selector' || first.length !== 1) {
      return;
    }

    if (((_first$first = first.first) === null || _first$first === void 0 ? void 0 : _first$first.type) !== 'class') return;
    var className = first.first.value;
    var replacement = (0, _camelCase["default"])(className);
    replacements.set(className, "$".concat(replacement));
    first.first.replaceWith(_postcssSelectorParser["default"].tag({
      value: replacement
    }));
  });
  var processSelector = (0, _postcssSelectorParser["default"])(function (root) {
    root.each(function (node) {
      var _node$at;

      if (node.type === 'selector' && ((_node$at = node.at(0)) === null || _node$at === void 0 ? void 0 : _node$at.value) !== '&') {
        node.insertBefore(node.at(0), _postcssSelectorParser["default"].combinator({
          value: ' '
        }));
        node.insertBefore(node.at(0), _postcssSelectorParser["default"].nesting({}));
      }
    });
    root.walkClasses(function (classNode) {
      if (!classNode.value) return;
      var replacement = replacements.get(classNode.value);

      if (replacement) {
        classNode.replaceWith(_postcssSelectorParser["default"].tag({
          value: replacement
        }));
      }
    });
  });
  var processSelectorInMediaOrGlobal = (0, _postcssSelectorParser["default"])(function (root) {
    root.walkClasses(function (classNode) {
      if (!classNode.value) return;
      var replacement = replacements.get(classNode.value);

      if (replacement) {
        classNode.replaceWith(_postcssSelectorParser["default"].tag({
          value: replacement
        }));
      }
    });
  });

  var processNode = function processNode(parentKey, node) {
    var globals = [];

    if (!parentKey || !/^@(keyframes|global)/.test(parentKey)) {
      node.properties = node.properties.filter(function (prop) {
        if (prop.type !== 'ObjectProperty') return true;
        var value = prop.value;
        var key = getRawKey(prop.key);
        if (!key || value.type !== 'ObjectExpression') return true;

        var _splitGlobals = splitGlobals(key),
            _splitGlobals2 = (0, _slicedToArray2["default"])(_splitGlobals, 2),
            localSelector = _splitGlobals2[0],
            globalSelector = _splitGlobals2[1];

        if (!parentKey) {
          if (key.startsWith('@')) return true;

          if (localSelector) {
            var newKey = processRootSelector.processSync(localSelector);
            if (newKey !== key) prop.key = objectPropertyKey(newKey);
          }
        } else {
          if (localSelector) {
            var replaced = parentKey.startsWith('@media') || parentKey === '@global' ? processSelectorInMediaOrGlobal.processSync(localSelector) : processSelector.processSync(localSelector);

            if (replaced !== key) {
              prop.key = objectPropertyKey(replaced);
            }
          }
        }

        if (globalSelector) {
          globals.push(_jscodeshift["default"].objectProperty(objectPropertyKey(processSelectorInMediaOrGlobal.processSync(globalSelector)), value));
        }

        return localSelector != null;
      });
    }

    if (globals.length) {
      node.properties.push(_jscodeshift["default"].objectProperty(_jscodeshift["default"].stringLiteral('@global'), _jscodeshift["default"].objectExpression(globals)));
    }

    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = node.properties[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        var prop = _step3.value;
        if (prop.type !== 'ObjectProperty') continue;
        var value = prop.value;
        var key = getRawKey(prop.key);
        if (!key || value.type !== 'ObjectExpression') continue;
        processNode(key, value);
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
          _iterator3["return"]();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }
  };

  processNode(null, root);
}

function convertAnimationNames(root, animationNames) {
  processDeep(function (node) {
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
      for (var _iterator4 = node.properties[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
        var prop = _step4.value;
        if (prop.type !== 'ObjectProperty') continue;
        var value = prop.value;

        switch (getRawKey(prop.key)) {
          case 'animationName':
            {
              if (value.type === 'StringLiteral' && animationNames.has(value.value)) {
                value.value = "$".concat(value.value);
              }

              break;
            }

          case 'animation':
            {
              if (value.type === 'StringLiteral') {
                value.value = value.value.replace(/\S+/g, function (word) {
                  return animationNames.has(word) ? "$".concat(word) : word;
                });
              }
            }
        }
      }
    } catch (err) {
      _didIteratorError4 = true;
      _iteratorError4 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion4 && _iterator4["return"] != null) {
          _iterator4["return"]();
        }
      } finally {
        if (_didIteratorError4) {
          throw _iteratorError4;
        }
      }
    }
  })(root);
}

var convertFallbacks = processDeep(function (node) {
  var properties = node.properties;
  var fallbacks = [];
  var encountered = new Set();

  for (var i = properties.length - 1; i >= 0; i--) {
    var prop = properties[i];
    if (prop.type !== 'ObjectProperty') continue;
    var key = getRawKey(prop.key);
    if (!key) continue;

    if (encountered.has(key)) {
      fallbacks.unshift(_jscodeshift["default"].objectExpression([prop]));
      properties.splice(i, 1);
    } else {
      encountered.add(key);
    }
  }

  if (fallbacks.length) properties.push(_jscodeshift["default"].objectProperty(_jscodeshift["default"].identifier('fallbacks'), _jscodeshift["default"].arrayExpression(fallbacks)));
});

function convertNodes(nodes) {
  var properties = [];
  var _iteratorNormalCompletion5 = true;
  var _didIteratorError5 = false;
  var _iteratorError5 = undefined;

  try {
    for (var _iterator5 = nodes[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
      var _node2 = _step5.value;

      switch (_node2.type) {
        case 'atrule':
          properties.push(convertAtRule(_node2));
          break;

        case 'rule':
          properties.push(convertRule(_node2));
          break;

        case 'decl':
          properties.push(convertDecl(_node2));
          break;
      }
    }
  } catch (err) {
    _didIteratorError5 = true;
    _iteratorError5 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion5 && _iterator5["return"] != null) {
        _iterator5["return"]();
      }
    } finally {
      if (_didIteratorError5) {
        throw _iteratorError5;
      }
    }
  }

  return _jscodeshift["default"].objectExpression(properties);
}

function convertAtRule(atrule) {
  var name = atrule.name,
      params = atrule.params,
      nodes = atrule.nodes;
  return _jscodeshift["default"].objectProperty(_jscodeshift["default"].stringLiteral(params ? "@".concat(name, " ").concat(params) : "@".concat(name)), convertNodes(nodes || []));
}

function convertRule(rule) {
  var selector = rule.selector,
      nodes = rule.nodes;
  var key = convertSelector(selector);
  var value = convertNodes(nodes || []);
  return _jscodeshift["default"].objectProperty(key, value);
}

function convertSelector(selector) {
  return _jscodeshift["default"].stringLiteral(selector);
}

var objectPropertyKey = function objectPropertyKey(s) {
  return /^[_a-z][_a-z0-9]*$/i.test(s) ? _jscodeshift["default"].identifier(s) : _jscodeshift["default"].stringLiteral(s);
};

function convertDeclValue(value) {
  value = value.replace(/^'|'$/g, '').replace(/^|$/g, '"').replace(/\\\\\\n/g, '\\n').replace(/url\('([^)]+)'\)/g, 'url($1)');
  return JSON.parse(value);
}

function convertDecl(decl) {
  var key = convertProp(decl.prop);
  var declValue = convertDeclValue(decl.value);
  var value;

  if (decl.prop === 'content') {
    value = _jscodeshift["default"].stringLiteral(JSON.stringify(declValue));
  } else {
    var defaultUnit = _defaultUnits["default"][decl.prop];
    value = _jscodeshift["default"].stringLiteral(declValue);

    if (defaultUnit) {
      var match = new RegExp("^(\\d+)".concat(defaultUnit, "$")).exec(declValue);
      if (match) value = _jscodeshift["default"].numericLiteral(parseFloat(match[1]));
    }
  }

  if (decl.important) {
    value = _jscodeshift["default"].arrayExpression([_jscodeshift["default"].arrayExpression([value]), _jscodeshift["default"].stringLiteral('!important')]);
  }

  return _jscodeshift["default"].objectProperty(key, value);
}

function convertProp(prop) {
  var result = (0, _camelCase["default"])(prop);
  return objectPropertyKey(/^-(?!ms-)/.test(prop) ? (0, _upperFirst["default"])(result) : result);
}